library(RWeka)
library(CORElearn)
library(xtable)

randomPartitionTVT <- function(noIns,noTrain,noVal,noTest){
	mix <- sample(1:noIns)
	
	train <- mix[1:as.integer(noTrain*noIns)]
	if(noVal==0.0)
		val <- train
	else
		val <- mix[(as.integer(noTrain*noIns)+1):(as.integer(noTrain*noIns)+as.integer(noVal*noIns))]
		
	test <- mix[(noIns-as.integer(noTest*noIns)+1):noIns]

	# Test always must be divisible by 10
	#newLen<-as.integer(length(test)/10)*10
	#sobran<-length(test)-newLen
	#quitar<-sample(1:length(test),sobran)
	#if(sobran>0)
	#	test<-test[-quitar]
	
 	
 	return(list(train,val,test))
}

# J48 Unpruned and with Laplace smoothing
learnJ48 <- function(train){
	model <- J48(class ~ .,train,control=Weka_control(U=TRUE,A=TRUE))
	return(model)
}

# Logistic Regression
learnLog <- function(train){
	model <- Logistic(class ~ .,train)
	return(model)
}

# IBk
learnIBk <- function(train,k,w){
	model <- IBk(class ~ .,train,control=Weka_control(K=k,I=w))
	return(model)
}

# Naive Bayes
NB <- make_Weka_classifier("weka/classifiers/bayes/NaiveBayes")
learnNB <- function(train){
	model <- NB(class ~ .,train)
	return(model)
}

predictClassProb <- function(model,test){
	return(list(class=predict(model,test,type="class"),probabilities=predict(model,test,type="probability")))
}

# Bayesian Model Averaging
BMA4 <- function(p1,p2,p3,p4){
	return((p1+p2+p3+p4)/4)	
}

BMA5 <- function(p1,p2,p3,p4,p5){
	return((p1+p2+p3+p4+p5)/5)	
}

# Bayesian Model Averaging (por clasificador)
BMA4w <- function(p1,p2,p3,p4,acc1,acc2,acc3,acc4){
	return((1/(acc1+acc2+acc3+acc4))*(p1*acc1+p2*acc2+p3*acc3+p4*acc4))
}

BMA5w <- function(p1,p2,p3,p4,p5,acc1,acc2,acc3,acc4,acc5){
	return((1/(acc1+acc2+acc3+acc4+acc5))*(p1*acc1+p2*acc2+p3*acc3+p4*acc4+p5*acc5))
}

# Bayesian Model Averaging (por clasificador y clase)
BMA4wj <- function(p1,p2,p3,p4,c1,c2,c3,c4,vacc1,vacc2,vacc3,vacc4){
	acc1<-vacc1[as.integer(c1)]
	acc2<-vacc2[as.integer(c2)]
	acc3<-vacc3[as.integer(c3)]
	acc4<-vacc4[as.integer(c4)]
	
	return((1/(acc1+acc2+acc3+acc4))*(p1*acc1+p2*acc2+p3*acc3+p4*acc4))
}

BMA5wj <- function(p1,p2,p3,p4,p5,c1,c2,c3,c4,c5,vacc1,vacc2,vacc3,vacc4,vacc5){
	acc1<-vacc1[as.integer(c1)]
	acc2<-vacc2[as.integer(c2)]
	acc3<-vacc3[as.integer(c3)]
	acc4<-vacc4[as.integer(c4)]
	acc5<-vacc5[as.integer(c5)]
	
	return((1/(acc1+acc2+acc3+acc4+acc5))*(p1*acc1+p2*acc2+p3*acc3+p4*acc4+p5*acc5))
}

# Classify function (Mayor probabilidad)
classify <- function(p){
	clase<-{}
	for(i in 1:length(p[,1])){
		clase<-c(clase,as.integer(which.max(p[i,])))
	}
	return(clase)	
}

#w<-w0.J48
#p<-p0.J48
classifyF <- function(w,p){
	clase<-{}
	for(i in 1:length(p[,1])){
		clase<-c(clase,as.integer(which.max(w*p[i,])))
	}
	return(clase)	
}




